// =========================================================================
// 1D-bar ME model of single-lap HBB joints with FGA adhesives
// code attached to the realted paper submitted to AIAA Journa
// author : Eric Paroissien
// date   : 07th January 2019
// LICENCE: CC BY NC 3.0 [https://creativecommons.org/licenses/by-nc/3.0/]
// =========================================================================
clc
clear all
// =========================================================================
// =========================================================================
// INPUT DATA
// =========================================================================
// Size of the problem
// =========================================================================
n = 479;                        // number of potential fasterner position
n_ME = n+1;                     // number of ME 
n_nodes = 2+2+(2*n_ME+2)-2;     // total number of nodes
n_K = 2*n+6;                    // total number of DoF
// =========================================================================
// Geometry
// =========================================================================
e = 0.2;                    // adhesive thickness in mm
e1 = 1.6;                   // adherend 1 thickness in mm
e2 = 1.6;                   // adherend 2 thickness in mm    
L = 48;                     // overlap length in mm
c = L/2;                    // half overlap in mm
bb = 24;                    // overlap width in mm
l1 = 48.;                   // length outside the overlap in mm
l2 = 48.;                   // length outside the overlap in mm
S1 = e1*bb;                 // cross section area of adherend 1 in mm²
S2 = e2*bb;                 // cross section area of adherend 2 in mm²
delta = zeros(n_ME,1);
for i = 1:n_ME
  delta(i,1) = L/n_ME;
end
abscissa=zeros(n_ME+1,1);
for i=2:n_ME+1
    abscissa(i,1)=abscissa(i-1)+delta(i-1);
end
// =========================================================================
// Material
// =========================================================================
E1=70000;                   // Young's modulus of adherend 1 in MPa
E2=70000;                   // Young's modulus of adherend 2 in MPa
CCf=30000.;                 // stiffness of fasteners in N/mm
CC=zeros(n,1);
n1=120;                     // position of the first fastener
n2=360;                     // position of the second fastener
CC(n1,1)=CCf;
CC(n2,1)=CCf;
// GRADUATION
// GRAD=0   =>  constant
// GRAD=100   =>  bilinear symmetrical from Ga_min to Ga_max to Ga_min & interpolation at the middle of ME
// GRAD=200   =>  power-p symmetrical from Ga_min to Ga_max to Ga_min & interpolation at the middle of ME
GRAD=0;
Ga_max=300;
Ga_min=1;
kII=zeros(n_ME,1);
abscissaINTERPOL=zeros(n_ME,1);
for i=1:n_ME
    abscissaINTERPOL(i,1)=abscissa(i,1)+delta(i,1)/2;
end
if GRAD==0 
    Ga = Ga_min;                  
    kIIa = Ga/e;                  
    kII=zeros(n_ME,1);
    for i=1:n_ME
        kII(i,1)=Ga/e;
    end
end
if GRAD==100               
    kIIa_max = Ga_max/e;        
    kIIa_min = Ga_min/e;              
    for i=1:n_ME/2
        kII(i,1)=kIIa_min+(kIIa_max-kIIa_min)*(abscissaINTERPOL(i,1))/c;
    end
    for i=n_ME/2+1:n_ME
        kII(i,1)=kIIa_max-(kIIa_max-kIIa_min)*(abscissaINTERPOL(i,1)-c)/c;
    end
end
if GRAD==200
    power=2;
    kIIa_max = Ga_max/e;        
    kIIa_min = Ga_min/e;              
    for i=1:n_ME/2
        kII(i,1)=kIIa_max-(kIIa_max-kIIa_min)*(((abscissaINTERPOL(i,1)-c))^power)/(c^power);
    end
    for i=n_ME/2+1:n_ME
        kII(i,1)=kIIa_max-(kIIa_max-kIIa_min)*(((abscissaINTERPOL(i,1)-c))^power)/(c^power);
    end
end
// =========================================================================
// Loading
// =========================================================================
force = 5000.;      // applied tensile force in N
// =========================================================================
// end of INPUT DATA
// =========================================================================
// =========================================================================

// =========================================================================
// DEDUCED CONSTANTS AND PARAMETERS
// =========================================================================
etavKRE = zeros(n_ME,1);
etav = zeros(n_ME,1);
ksiKRE = zeros(n_ME,1);
etaKRE = zeros(n_ME,1);
eta = zeros(n_ME,1);
psi = zeros(n_ME,1);
teta = zeros(n_ME,1);
omega = zeros(n_ME,1);
for i=1:n_ME
    etavKRE(i,1) = kII(i,1)*(1/e1/E1+1/e2/E2);
    etav(i,1) = sqrt(etavKRE(i,1));
    ksiKRE(i,1) = 0;    
    etaKRE(i,1) = etavKRE(i,1)/(1+ksiKRE(i,1));
    eta(i,1) = sqrt(etaKRE(i,1));
    psi(i,1) = kII(i,1)*((1/e1)/E1-(1/e2)/E2)/(1+ksiKRE(i,1));
    teta(i,1) = 1+ psi(i,1)/etaKRE(i,1);
    omega(i,1) = 1-psi(i,1)/etaKRE(i,1);
end
lambdat = e1*E1/(e2*E2);
// =========================================================================
// end of DEDUCED CONSTANTS AND PARAMETERS
// =========================================================================


// =========================================================================
// ELEMENTARY STIFFNESS MATRIX
// =========================================================================
// ME stiffness matrix
// =========================================================================
K_BC = zeros(4,4,n_ME);
for inc=1:n_ME
    kappa = 0.5*E2*S2/delta(inc,1);
    cs = eta(inc,1)*delta(inc,1)/sinh(eta(inc,1)*delta(inc,1));
    ct = eta(inc,1)*delta(inc,1)*coth(eta(inc,1)*delta(inc,1));
    Kbc = zeros(4,4);
    Kbc(1,1) = omega(inc,1)*kappa*(ct+lambdat);
    Kbc(1,2) = omega(inc,1)*kappa*(1-ct);
    Kbc(1,3) = omega(inc,1)*kappa*(-cs-lambdat);
    Kbc(1,4) = omega(inc,1)*kappa*(cs-1);
    Kbc(2,2) = omega(inc,1)*kappa*(ct+1/lambdat);
    Kbc(2,3) = Kbc(1,4);
    Kbc(2,4) = omega(inc,1)*kappa*(-cs-1/lambdat);
    Kbc(3,3) = Kbc(1,1);
    Kbc(3,4) = Kbc(1,2);
    Kbc(2,1) = Kbc(1,2);
    Kbc(3,1) = Kbc(1,3);
    Kbc(3,2) = Kbc(2,3);
    Kbc(4,1) = Kbc(1,4);
    Kbc(4,2) = Kbc(2,4);
    Kbc(4,3) = Kbc(3,4);
    Kbc(4,4) = Kbc(2,2);
    K_BC(1:4,1:4,inc) = Kbc(1:4,1:4);
end
// =========================================================================
// BE stiffness matrix
// =========================================================================
kb1 = E1*S1/l1;
kb2 = E2*S2/l2;
// =========================================================================
// end of ELEMENTARY STIFFNESS MATRIX
// =========================================================================

// =========================================================================
// INITIAL STIFFNESS MATRIX
// =========================================================================
K = zeros(2*n+6,2*n+6);
for inc=1:n_ME
    K(2*inc:2*inc+3,2*inc:2*inc+3) = K(2*inc:2*inc+3,2*inc:2*inc+3)+K_BC(1:4,1:4,inc);
end
K(1,1) = kb1;
K(1,2) = -kb1;
K(2,1) = -kb1;
K(2,2) = K(2,2)+kb1;
K(2*n+5,2*n+5) = K(2*n+5,2*n+5)+kb2;
K(2*n+5,2*n+6) = -kb2;
K(2*n+6,2*n+5) = -kb2;
K(2*n+6,2*n+6) = kb2;
for i = 1:n
  K(2*i+2,2*i+2) = K(2*i+2,2*i+2)+CC(i,1);
  K(2*i+2,2*i+3) = K(2*i+2,2*i+3)-CC(i,1);
  K(2*i+3,2*i+2) = K(2*i+3,2*i+2)-CC(i,1);
  K(2*i+3,2*i+3) = K(2*i+3,2*i+3)+CC(i,1);
end
// =========================================================================
// end of INITIAL STIFFNESS MATRIX
// =========================================================================

// =========================================================================
// BOUNDARY CONDITIONS
// =========================================================================
// =========================================================================
// Augmented Problem
// =========================================================================
maxAUG = 2*n+6+1;
CLdep = zeros(1,2*n+6);
CLdep(1,1) = 1; 
Kaug = zeros(maxAUG,maxAUG);
Kaug(1:2*n+6,1:2*n+6) = K;
Kaug(1:2*n+6,2*n+6+1:2*n+6+1) = CLdep';
Kaug(2*n+6+1:2*n+6+1,1:2*n+6) = CLdep;
qL = zeros(1,1);
FL = zeros(maxAUG,1);
FL(2*n+6,1) = FL(2*n+6,1)+force;
FL(2*n+6+1:2*n+6+1,1) = qL;
// =========================================================================
// end of BOUNDARY CONDITIONS
// =========================================================================
// =========================================================================
// COMPUTATION
// =========================================================================
XL = Kaug\FL;
X=XL(1:2*n+6,1);
// =========================================================================
// end of COMPUTATION
// ========================================================================
// =========================================================================
// POST-PROCESSSING
// =========================================================================
// Integration constants
// =========================================================================
c1 = zeros(n_ME,1);
c2 = zeros(n_ME,1);
c3 = zeros(n_ME,1);
c4 = zeros(n_ME,1);
for i = 1:n_ME
  c1(i,1) = 1/(2*sinh(eta(i,1)*delta(i,1)))*((X(2*i+1,1)-X(2*i,1))*exp(eta(i,1)*delta(i,1))+X(2*i+2,1)-X(2*i+3,1));
  c2(i,1) = 1/(2*sinh(eta(i,1)*delta(i,1)))*((X(2*i,1)-X(2*i+1,1))*exp(-eta(i,1)*delta(i,1))+X(2*i+3,1)-X(2*i+2,1));
  c3(i,1) = 1/delta(i,1)*(teta(i,1)*X(2*i+3,1)+omega(i,1)*X(2*i+2,1)-teta(i,1)*X(2*i+1,1)-omega(i,1)*X(2*i,1));
  c4(i,1) = teta(i,1)*X(2*i+1,1)+omega(i,1)*X(2*i,1);
end
// =========================================================================
// nodal longitudinal dispalcements
// =========================================================================
u1 = zeros(n_ME+1,1);
u2 = zeros(n_ME+1,1);
for i=1:n_ME+1
    u1(i,1)=X(2*i,1);
    u2(i,1)=X(2*i+1,1);
end
x_specimen=X(2*n+6,1)
K_specimen=force/x_specimen
// =========================================================================
// Nodal forces
// =========================================================================
QQ = zeros(4*n+4,1);
for i = 1:n+1
  QQ(4*i-3,1) = -0.5*E1*S1*(-teta(i,1)*eta(i,1)*(c2(i,1)-c1(i,1))+c3(i,1));
  QQ(4*i-2,1) = -0.5*E2*S2*(omega(i,1)*eta(i,1)*(c2(i,1)-c1(i,1))+c3(i,1));
  QQ(4*i-1,1) = 0.5*E1*S1*(-teta(i,1)*eta(i,1)*(c2(i,1)*exp(eta(i,1)*delta(i,1))-c1(i,1)*exp(-eta(i,1)*delta(i,1)))+c3(i,1));
  QQ(4*i,1) = 0.5*E2*S2*(omega(i,1)*eta(i,1)*(c2(i,1)*exp(eta(i,1)*delta(i,1))-c1(i,1)*exp(-eta(i,1)*delta(i,1)))+c3(i,1));
end
NormalForce1 = zeros(n_ME+1,1);
NormalForce2 = zeros(n_ME+1,1);
for i=1:n_ME
    NormalForce1(i,1)=-QQ(4*i-3,1);
    NormalForce2(i,1)=-QQ(4*i-2,1);
end
NormalForce1(n_ME+1,1)=QQ(4*(n+1)-1,1);
NormalForce2(n_ME+1,1)=QQ(4*(n+1),1);
// =========================================================================
// Adhesive shear stress and strain
// =========================================================================
kIIpost=zeros(n_ME+1,1);
for i=1:n_ME/2
    kIIpost(i,1)=kII(i,1)/(1+ksiKRE(i,1));
end
kIIpost(n_ME/2+1,1)=0.5*(kII(n_ME/2,1)/(1+ksiKRE(n_ME/2,1))+kII(n_ME/2+1,1)/(1+ksiKRE(n_ME/2+1,1)));
for i=n_ME/2+2:n_ME+1
    kIIpost(i,1)=kII(i-1,1)/(1+ksiKRE(i-1,1));
end
ShearStress=zeros(n_ME+1,1);
ShearStrain=zeros(n_ME+1,1);
DensityEnergy=zeros(n_ME+1,1);
for i=1:n_ME+1
    ShearStress(i,1)=kIIpost(i,1)*(u2(i,1)-u1(i,1));
    ShearStrain(i,1)=(u2(i,1)-u1(i,1))/e;
    DensityEnergy(i,1)=0.5*e*ShearStrain(i,1)*ShearStress(i,1);
end
T_max=max(ShearStress);
T_edge=ShearStress(1,1);
gam_max=max(ShearStrain);
gam_edge=ShearStrain(1,1);
U_max=max(DensityEnergy);
U_edge=DensityEnergy(1,1);
// =========================================================================
// Load transfer rate
// =========================================================================
tau = zeros(n,1);
for i = 1:n
  tau(i,1) = CC(i,1)/force*(X(2*i+3,1)-X(2*i+2,1));
end
tau_fix=tau(n1,1)
// =========================================================================
// Curves
// =========================================================================
subplot(5,1,1)
plot(abscissa,ShearStress)
subplot(5,1,2)
plot(abscissa,ShearStrain)
subplot(5,1,3)
plot(abscissa,DensityEnergy)
subplot(5,1,4) 
plot(abscissa,NormalForce2)
subplot(5,1,5) 
plot(abscissaINTERPOL,kII)





